@@ -25,14 +25,58 @@ import scipy.interpolate as sp_int
 
 
 class GeodesicEngine():
+    r"""This is the main symbolic tool within PyGRO to perform tensorial calculations
+    starting from the spacetime metric. The ``Metric`` object can be initialized in two separate ways:
 
-    def __init__(self, metric, verbose = True, integrator = "dp45"):
+    .. rubric:: Interactive mode
+    
+    In this case the ``Metric`` has to be initialized without any additional argument.
+    Later the :py:func:`Metric.initialize_metric` method should be called without additional arguments in order to enter the interactive mode.
+    During this phase the user will be asked to input:
 
-        self.link_metrics(metric, verbose)
+    * The ``name`` of the spacetime.
+    * The symbolic ``coordinates`` in which the metric is expressed.
+    * The symbolic expression of the spacetime metric which can be either expressed as a line elemnt :math:`ds^2 = g_{\mu\nu}dx^\mu dx^\nu` or as the single components :math:`g_{\mu\nu}` of the metric tensor.
+    * The symbolix expression of ``transformation functions`` to pseudo-cartesian coordinates which can be useful to :doc:`visualize`.
+
+    After the insertion of all the required information the metric is initialized.
+
+    .. rubric:: Functional mode
+
+    In this case the ``Metric`` object is initialized without interactive input by the user. The arguments to build the metric can be either passed to the ``Metric`` constructor upon class initialization,
+    
+    >>> spacetime_metric = pygro.Metric(**kwargs)
+
+    or as arguments to the  :py:func:`Metric.initialize_metric` method:
+
+    >>> spacetime_metric = pygro.Metric()
+    >>> spacetime_metric.initialize_metric(**kwargs)
+
+    The ``**kwargs`` that can be passed to initialize a ``Metric`` object are the following:
+
+    :param name: The name of the metric to initialize.
+    :type name: str
+    :param coordinates: Four-dimensional list containing the symbolic expression of the space-time coordinates in which the `line_element` argument is written.
+    :type coordinates: list of str
+    :param line_element: A string containing the symbolic expression of the line element (that will be parsed using `sympy`) expressed in the space-time coordinates defined in the `coordinates` argument.
+    :type line_element: str
+
+    .. note::
+        Test
+    
+    After successful initialization, the ``Metric`` instance has the following attributes:
+
+    :ivar g: The symbolic representation of the metric tensor. It is a :math:`4\times4` ``sympy.Matrix`` object.
+    :type g: sympy.Matric
+
+    """
+    def __init__(self, metric, verbose = True, backend = "autowrap", integrator = "dp45"):
+
+        self.link_metrics(metric, verbose, backend)
         self.integrator = integrator
         self.wrapper = None
     
-    def link_metrics(self, g, verbose):
+    def link_metrics(self, g, verbose = "True", backend = "autowrap"):
         if verbose:
             print("Linking {} to the Geodesic Engine".format(g.name))
 
@@ -46,7 +90,7 @@ class GeodesicEngine():
         self.metric.geodesic_engine_linked = True
         self.metric.geodesic_engine = self
 
-        if len(self.metric.get_parameters_functions()) > 0:
+        if (backend == "lambdify") or len(self.metric.get_parameters_functions()) > 0:
             self.wrapper = "lambdify"
         else:
             self.wrapper = "autowrap"
@@ -59,8 +103,24 @@ class GeodesicEngine():
             else:
                 self.motion_eq_f.append(lambdify([*self.metric.x, *self.metric.u, *self.metric.get_parameters_symb()], self.metric.subs_functions(eq)))
 
-        self.evaluate_parameters()
+        def f_eq(tau, xu):
+            return np.array([self.motion_eq_f[i](*xu, *self.metric.get_parameters_val()) for i in range(8)])
 
+        self.motion_eq = f_eq
+        
+        u0_f_timelike = lambdify([*self.metric.x, self.metric.u[1], self.metric.u[2], self.metric.u[3], *self.metric.get_parameters_symb()], self.metric.subs_functions(g.u0_s_timelike))
+        
+        def f_u0_timelike(initial_x, u1, u2, u3):
+            return abs(u0_f_timelike(*initial_x, u1, u2, u3, *self.metric.get_parameters_val()))
+
+        u0_f_null = lambdify([*self.metric.x, self.metric.u[1], self.metric.u[2], self.metric.u[3], *self.metric.get_parameters_symb()], self.metric.subs_functions(g.u0_s_null))
+
+        def f_u0_null(initial_x, u1, u2, u3):
+            return abs(u0_f_null(*initial_x, u1, u2, u3, *self.metric.get_parameters_val()))
+
+        self.u0_f_timelike = f_u0_timelike
+        self.u0_f_null = f_u0_null
+        
         if verbose:
             print("Metric linking complete.")
 
@@ -111,20 +171,7 @@ class GeodesicEngine():
 
         if interpolate == True:
             geo.x_int = sp_int.interp1d(geo.tau, geo.x, axis = 0, kind = "cubic")
-            geo.u_int = sp_int.interp1d(geo.tau, geo.u, axis = 0, kind = "cubic")
-    
-    def evaluate_parameters(self):
-
-        def f(tau, xu):
-            return np.array([self.motion_eq_f[i](*xu, *self.metric.get_parameters_val()) for i in range(8)])
-
-        self.motion_eq = f
-        
-        u0_timelike = self.metric.evaluate_parameters(self.u0_s_timelike)
-        u0_null = self.metric.evaluate_parameters(self.u0_s_null)
-
-        self.u0_f_null = sp.lambdify([self.metric.x, self.metric.u[1], self.metric.u[2], self.metric.u[3]], u0_null, 'numpy')
-        self.u0_f_timelike = sp.lambdify([self.metric.x, self.metric.u[1], self.metric.u[2], self.metric.u[3]], u0_timelike, 'numpy')
+            geo.u_int = sp_int.interp1d(geo.tau, geo.u, axis = 0, kind = "cubic")        
     
     def set_integrator(self, integrator):
         self.integrator = integrator
\ No newline at end of file
