@@ -21,78 +21,186 @@ from sympy.core.function import AppliedUndef
 #                                                                                       #
 #########################################################################################
 
+def parse_expr(expr):
+    return sp.parse_expr(expr)
+
 class Metric():
-    def __init__(self):
+    r"""This is the main symbolic tool within PyGRO to perform tensorial calculations
+    starting from the spacetime metric. The ``Metric`` object can be initialized in two separate ways:
+
+    .. rubric:: Interactive mode
+    
+    In this case the ``Metric`` has to be initialized without any additional argument.
+    Later the :py:func:`Metric.initialize_metric` method should be called without additional arguments in order to enter the interactive mode.
+    During this phase the user will be asked to input:
+
+    * The ``name`` of the spacetime.
+    * The symbolic ``coordinates`` in which the metric is expressed.
+    * The symbolic expression of the spacetime metric which can be either expressed as a line elemnt :math:`ds^2 = g_{\mu\nu}dx^\mu dx^\nu` or as the single components :math:`g_{\mu\nu}` of the metric tensor.
+    * The symbolix expression of ``transformation functions`` to pseudo-cartesian coordinates which can be useful to :doc:`visualize`.
+
+    After the insertion of all the required information the metric is initialized.
+
+    .. rubric:: Functional mode
+
+    In this case the ``Metric`` object is initialized without interactive input by the user. The arguments to build the metric can be either passed to the ``Metric`` constructor upon class initialization,
+    
+    >>> spacetime_metric = pygro.Metric(**kwargs)
+
+    or as arguments to the  :py:func:`Metric.initialize_metric` method:
+
+    >>> spacetime_metric = pygro.Metric()
+    >>> spacetime_metric.initialize_metric(**kwargs)
+
+    The ``**kwargs`` that can be passed to initialize a ``Metric`` object are the following:
+
+    :param name: The name of the metric to initialize.
+    :type name: str
+    :param coordinates: Four-dimensional list containing the symbolic expression of the space-time coordinates in which the `line_element` argument is written.
+    :type coordinates: list of str
+    :param line_element: A string containing the symbolic expression of the line element (that will be parsed using `sympy`) expressed in the space-time coordinates defined in the `coordinates` argument.
+    :type line_element: str
+
+    .. note::
+        Test
+    
+    After successful initialization, the ``Metric`` instance has the following attributes:
+
+    :ivar g: The symbolic representation of the metric tensor. It is a :math:`4\times4` ``sympy.Matrix`` object.
+    :type g: sympy.Matric
+
+    """
+    def __init__(self, **kwargs):
         self.initialized = 0
         self.initialized_metric = 0
         self.geodesic_engine_linked = False
-    
-    def initialize_metric(self):
-        self.name = input("Insert the name of the spacetime (e.g. 'Kerr metric'): ")
 
-        print("Initialazing {}".format(self.name))
-        
-        print("Define coordinates symbols:")
+        if len(kwargs) > 0:
+            if "load" in kwargs:
+                self.load_metric(kwargs["load"])
+            self.initialize_metric(**kwargs)
+    
+    def initialize_metric(self, **kwargs):
+        """Initializes the `Metric` either in *Interactive* or *Functional* mode. 
+
+        :param \**kwargs:
+            to pass only to access *Functional* mode. In this case, required parameters are ``name``, ``coordinates`` and ``line_element``,
+            but additional parameters can and *must* be passed on occasions (see :doc:`create_metric` to see examples). 
+        """
+
         self.x = []
         self.dx = []
         self.x_str = []
         self.u = []
-        for i in range(4):
-            coordinate = input("Coordinate {}: ".format(i))
-            setattr(self, coordinate, sp.symbols(coordinate))
-            self.x.append(self.__dict__[coordinate])
-            self.x_str.append(coordinate)
 
-            velocity = "u_" + coordinate
-            setattr(self, velocity, sp.symbols(velocity))
-            self.u.append(self.__dict__[velocity])
+        interactive_metric_insertion = len(kwargs) == 0        
 
-            differential = "d" + coordinate
-            setattr(self, differential, sp.symbols(differential))
-            self.dx.append(self.__dict__[differential])
-        
+        if not interactive_metric_insertion:
 
-        case = input("From? [tensor/line element]: ")
+            minimal_kwargs = ["name", "coordinates", "line_element"]
 
-        if case == "tensor":
-            print("Define metric tensor components:")
+            for kwarg in minimal_kwargs:
+                if not kwarg in kwargs:
+                    raise ValueError('initialize_metric in non-interactive mode should have "name", "coordinates", "line_element" as minimal arguments')
+
+            self.name = kwargs['name']
+
+            coordinates_input = kwargs["coordinates"]
+            
+            if len(coordinates_input) != 4:
+                raise ValueError('coordinates should be a 4-dimensional list of strings')
+            
+            for coordinate in coordinates_input:
+                setattr(self, coordinate, sp.symbols(coordinate))
+                self.x.append(self.__dict__[coordinate])
+                self.x_str.append(coordinate)
+
+                velocity = "u_" + coordinate
+                setattr(self, velocity, sp.symbols(velocity))
+                self.u.append(self.__dict__[velocity])
+
+                differential = "d" + coordinate
+                setattr(self, differential, sp.symbols(differential))
+                self.dx.append(self.__dict__[differential])
+            
             self.g = sp.zeros(4, 4)
             self.g_str = np.zeros([4,4], dtype = object)
 
-            for i in range(4):
-                for j in range(4):
-                    while True:
-                        try:
-                            component = input("g[{},{}]: ".format(i, j))
-                            component_symb = sp.parse_expr(component)
-                        except:
-                            print("Please insert a valid expression for the component.")
-                            continue
-                        else:
-                            self.g[i,j] = component_symb
-                            self.g_str[i,j] = component
-                            break
-                    
-        elif case == "line element":
-            self.g = sp.zeros(4, 4)
-            self.g_str = np.zeros([4,4], dtype = object)
-            while True:
-                try:
-                    ds2_str = input("ds^2 = ")
-                    ds2_sym = sp.expand(sp.parse_expr(ds2_str))
-                except:
-                    print("Please insert a valid expression for the line element.")
-                    continue
-                else:
-                    self.ds2 = ds2_sym
-                    for i, dx1 in enumerate(self.dx):
-                        for j, dx2 in enumerate(self.dx):
-                            self.g[i,j] = self.ds2.coeff(dx1*dx2,1)
-                            self.g_str[i,j] = str(self.g[i,j])
-                    break
+            try:
+                ds2_str = kwargs["line_element"]
+                ds2_sym = sp.expand(sp.parse_expr(ds2_str))
+            except:
+                raise ValueError("Please insert a valid expression for the line element.")
+            else:
+                self.ds2 = ds2_sym
+                for i, dx1 in enumerate(self.dx):
+                    for j, dx2 in enumerate(self.dx):
+                        self.g[i,j] = self.ds2.coeff(dx1*dx2,1)
+                        self.g_str[i,j] = str(self.g[i,j])
+
         else:
-            raise("Only 'tensor' or 'line element' are accepted method for parsing the metric.")
-        
+            self.name = input("Insert the name of the spacetime (e.g. 'Kerr metric'): ")
+
+            print("Initialazing {}".format(self.name))
+            
+            print("Define coordinates symbols:")
+            
+            for i in range(4):
+                coordinate = input("Coordinate {}: ".format(i))
+                setattr(self, coordinate, sp.symbols(coordinate))
+                self.x.append(self.__dict__[coordinate])
+                self.x_str.append(coordinate)
+
+                velocity = "u_" + coordinate
+                setattr(self, velocity, sp.symbols(velocity))
+                self.u.append(self.__dict__[velocity])
+
+                differential = "d" + coordinate
+                setattr(self, differential, sp.symbols(differential))
+                self.dx.append(self.__dict__[differential])
+            
+
+            case = input("From? [tensor/line element]: ")
+
+            if case == "tensor":
+                print("Define metric tensor components:")
+                self.g = sp.zeros(4, 4)
+                self.g_str = np.zeros([4,4], dtype = object)
+
+                for i in range(4):
+                    for j in range(4):
+                        while True:
+                            try:
+                                component = input("g[{},{}]: ".format(i, j))
+                                component_symb = sp.parse_expr(component)
+                            except:
+                                print("Please insert a valid expression for the component.")
+                                continue
+                            else:
+                                self.g[i,j] = component_symb
+                                self.g_str[i,j] = component
+                                break
+                        
+            elif case == "line element":
+                self.g = sp.zeros(4, 4)
+                self.g_str = np.zeros([4,4], dtype = object)
+                while True:
+                    try:
+                        ds2_str = input("ds^2 = ")
+                        ds2_sym = sp.expand(sp.parse_expr(ds2_str))
+                    except:
+                        print("Please insert a valid expression for the line element.")
+                        continue
+                    else:
+                        self.ds2 = ds2_sym
+                        for i, dx1 in enumerate(self.dx):
+                            for j, dx2 in enumerate(self.dx):
+                                self.g[i,j] = self.ds2.coeff(dx1*dx2,1)
+                                self.g_str[i,j] = str(self.g[i,j])
+                        break
+            else:
+                raise("Only 'tensor' or 'line element' are accepted method for parsing the metric.")
+            
         print("Calculating inverse metric...")
         self.g_inv = self.g.inv()
         self.g_inv_str = np.zeros([4,4], dtype = object)
@@ -147,17 +255,34 @@ class Metric():
         if len(free_sym) > 0:
             for sym in free_sym:
                 self.add_parameter(str(sym))
-                value = float(input("Insert value for {}: ".format(str(sym))))
-                self.set_constant(**{str(sym): value})
+                if str(sym) in kwargs:
+                    self.set_constant(**{str(sym): kwargs.get(str(sym))})
+                else:
+                    value = float(input("Insert value for {}: ".format(str(sym))))
+                    self.set_constant(**{str(sym): value})
 
         if len(free_func) > 0:
             for func in free_func:
                 self.add_parameter(str(func))
-                kind = input("Define kind for function {} [expr/py]: ".format(str(func)))
+                
+                if interactive_metric_insertion:
+                    kind = input("Define kind for function {} [expr/py]: ".format(str(func)))
+                else:
+                    if not str(func.func) in kwargs:
+                        raise ValueError("Auxiliary functions shoudld be passed as arguments, either as strings (expression mode) or as python methods (functional mode).")
+                    
+                    if isinstance(kwargs[str(func.func)], str):
+                        kind = "expr"
+                    elif callable(kwargs[str(func.func)]):
+                        kind = "py"
 
                 if kind == "expr":
                     self.parameters[str(func)]['kind'] = "expression"
-                    expr_str = input("{} = ".format(str(func)))
+                    if interactive_metric_insertion:
+                        expr_str = input("{} = ".format(str(func)))
+                    else:
+                        expr_str = kwargs[str(func.func)]
+
                     expr_sym = sp.expand(sp.parse_expr(expr_str))
 
                     self.parameters[str(func)]['value'] = expr_sym
@@ -166,56 +291,95 @@ class Metric():
 
                     for sym in expr_free_sym:
                         self.add_parameter(str(sym))
-                        value = float(input("Insert value for {}: ".format(str(sym))))
-                        self.set_constant(**{str(sym): value})
-
+                        if str(sym) in kwargs:
+                            self.set_constant(**{str(sym): kwargs.get(str(sym))})
+                        else:
+                            value = float(input("Insert value for {}: ".format(str(sym))))
+                            self.set_constant(**{str(sym): value})
 
                     for arg in list(func.args):
                         self.parameters[str(func)][f"d{str(func.func)}d{str(arg)}"] = expr_sym.diff(arg)
 
-
                 elif kind == "py":
 
                     self.parameters[str(func)]['kind'] = "pyfunc"
 
-                    print(f"Remember to set the Python function for {str(func)} and its derivatives with")
+                    if interactive_metric_insertion:
+                        print(f"Remember to set the Python function for {str(func)} and its derivatives with")
                     
-                    derlist = ""
-                    for arg in list(func.args):
-                        derlist += f", d{str(func.func)}d{str(arg)} = func"
+                        derlist = ""
+                        for arg in list(func.args):
+                            derlist += f", d{str(func.func)}d{str(arg)} = func"
+                        
+                        print(f"set_function_to_parameter({str(func)}, f = func{derlist})")
+
+                    else:
+                        args = list(func.args)
+                        derlist = [f"d{str(func.func)}d{str(arg)}" for arg in args]
+
+                        arg_derlist = {}
+
+                        for der in derlist:
+                            if not der in kwargs:
+                                raise ValueError(f"In functional mode pass as arguments functional derivatives  of auxiliary functions. In this case '{der}' is missing.")
+                            else:
+                                arg_derlist[der] = kwargs[der]
+
+                        self.set_function_to_parameter(str(func), kwargs[str(func.func)], **arg_derlist)
+
+            if interactive_metric_insertion:
+                while True:
+                    case = input("Want to insert transform functions to pseudo-cartesian coordiantes? [y/n] ")
+
+                    if case == "y":
+                        for x in ["t", "x", "y", "z"]:
+                            while True:
+                                try:
+                                    x_inpt = input(f"{x} = ")
+                                    x_symb = sp.parse_expr(x_inpt)
+                                except KeyboardInterrupt:
+                                    raise SystemExit
+                                except:
+                                    print("Insert a valid expression.")
+                                    continue
+                                else:
+                                    self.transform_s.append(x_symb)
+                                    self.transform_functions_str.append(x_inpt)
+                                    self.transform_functions.append(sp.lambdify([self.x], self.evaluate_parameters(x_symb), 'numpy'))
+                                    break
+                        break
+                                    
+                    elif case == "n":
+                        break
+                    else:
+                        print("Not a valid input.")
+                        continue
+            else:
+                if "transform" in kwargs:
+                    transform = kwargs["transform"]
+                    if len(transform) != 4:
+                        raise ValueError('"transform" should be a 4-dimensional list of strings')
                     
-                    print(f"set_function_to_parameter({str(func)}, f = func{derlist})")
-
-        while True:
-            case = input("Want to insert transform functions to pseudo-cartesian coordiantes? [y/n] ")
-
-            if case == "y":
-                for x in ["t", "x", "y", "z"]:
-                    while True:
+                    for i, x in enumerate(["t", "x", "y", "z"]):
                         try:
-                            x_inpt = input(f"{x} = ")
+                            x_inpt = transform[i]
                             x_symb = sp.parse_expr(x_inpt)
-                        except KeyboardInterrupt:
-                            raise SystemExit
                         except:
-                            print("Insert a valid expression.")
-                            continue
+                            raise ValueError(f"Insert a valid expression for transform function {x}")
                         else:
                             self.transform_s.append(x_symb)
                             self.transform_functions_str.append(x_inpt)
                             self.transform_functions.append(sp.lambdify([self.x], self.evaluate_parameters(x_symb), 'numpy'))
-                            break
-                break
-                            
-            elif case == "n":
-                break
-            else:
-                print("Not a valid input.")
-                continue
+                    
 
         print("The metric_engine has been initialized.")
 
     def save_metric(self, filename):
+        r"""Saves the metric into a *.metric* file which can later be loaded with the :py:func:`Metric.load_metric` method.
+
+        :param filename: The name of the *.metric* file in which to save the metric.
+        :type filename: str
+        """
         if self.initialized:
             with open(filename, "w+") as file:
                 output = {}
@@ -252,7 +416,11 @@ class Metric():
             print("Initialize (initialize_metric) or load (load_metric) a metric before saving.")
     
     def load_metric(self, filename, verbose = True, **params):
+        r"""Loads the metric from a *.metric* file which has been saved through the :py:func:`Metric.save_metric` method.
 
+        :param filename: The name of the *.metric* file from which to load the metric.
+        :type filename: str
+        """
         f = open(filename, "r")
 
         load = json.load(f)
@@ -267,6 +435,12 @@ class Metric():
 
         self.name = load['name']
 
+        if not "expressions" in load:
+            load["expressions"] = []
+        
+        if not "functions" in load:
+            load["functions"] = []
+
         if verbose:
             print("Loading {}".format(self.name))
         
@@ -396,13 +570,16 @@ class Metric():
         if verbose:
             print("The metric_engine has been initialized.")
 
-    def add_parameter(self, symbol):
+    def add_parameter(self, symbol, value = None):
         if self.initialized:
             self.parameters[symbol] = {}
             self.parameters[symbol]['symbol'] = symbol
             self.parameters[symbol]['symbolic'] = sp.parse_expr(symbol)
-            self.parameters[symbol]['value'] = None
-            self.parameters[symbol]['kind'] = None
+            self.parameters[symbol]['value'] = value
+            if value != None:
+                self.parameters[symbol]['kind'] = "constant"
+            else:
+                self.parameters[symbol]['kind'] = None
         else:
             print("Initialize (initialize_metric) or load (load_metric) a metric before adding parameters.")
     
@@ -419,6 +596,14 @@ class Metric():
             print("Initialize (initialize_metric) or load (load_metric) a metric before adding parameters.")
     
     def set_expression_to_parameter(self, param, expr_str):
+        r"""Selects the ``param`` element from the ``metric.parameters`` dictionary, sets its kind to ``"expr"`` and assignes to it a value which is the sybolic parsed
+        espresion in the `expr_str` argument.
+        
+        :param param: The symbolic name of the parameter to modify.
+        :type param: str
+        :param expr_str: The symbolic expresison to assign to this parameter.
+        :type expr_str: str
+        """
         if self.initialized:
             if param in self.parameters:
                 self.parameters[str(param)]['value'] = sp.parse_expr(expr_str)
@@ -447,6 +632,22 @@ class Metric():
 
 
     def set_function_to_parameter(self, param, function, **derivatives):
+        r"""Selects the ``param`` element from the ``metric.parameters`` dictionary, sets its kind to ``"py"`` and assignes as its value
+        the ``function`` method that is passed as argument. 
+
+        .. note::
+            The ``function`` argument **must** be a method which admits four floats and returns a single float as results.
+            However, in the ``line_element`` argument, upon initialization of the ``Metric``, only explicit coordinate dependences of the
+            functional parameter must be indicated. Moreover, for each of the explicit coordinate dependence of the function its derivative should also
+            be passed in argument
+
+            :Example:
+        
+        :param param: The symbolic name of the parameter to modify.
+        :type param: str
+        :param expr_str: The symbolic expresison to assign to this parameter.
+        :type expr_str: str
+        """
         if self.initialized:
             
             args = list(self.parameters[str(param)]['symbolic'].args)
@@ -464,6 +665,8 @@ class Metric():
         else:
             print("Initialize (initialize_metric) or load (load_metric) a metric before adding parameters.")
 
+    # Getters
+
     def get_parameters_symb(self):
         return [self.parameters[constant]['symbolic'] for constant in self.parameters if self.parameters[constant]['kind'] == "constant"]
 
@@ -478,6 +681,16 @@ class Metric():
 
     def get_parameters_functions(self):
         return {self.parameters[constant]['symbol']: self.parameters[constant] for constant in self.parameters if self.parameters[constant]["kind"] == "pyfunc"}
+    
+    def get_constant(self, parameter):
+        """Returns the value of the constant ``parameter``, if defined.
+        """
+        if self.parameters[parameter]["kind"] == "constant":
+            return self.parameters[parameter]["value"]
+        else:
+            raise TypeError(f"Unknown constant {parameter}")
+
+    # Auxiliary functions
 
     def subs_functions(self, expr):
         functions = self.get_parameters_expressions()
@@ -493,7 +706,7 @@ class Metric():
                 
                 expr = expr.subs(subs)
             
-            expr = expr.subs(f, functions[func]['value'])
+                expr = expr.subs(f, functions[func]['value'])
         
         functions = self.get_parameters_functions()
 
@@ -508,7 +721,7 @@ class Metric():
                 
                 expr = expr.subs(subs)
             
-            expr = expr.subs(f, functions[func]['value'](*self.x))
+                expr = expr.subs(f, functions[func]['value'](*self.x))
         
         return expr
 
@@ -542,6 +755,9 @@ class Metric():
                         self.transform_s.append(x_symb)
                         self.transform_functions.append(sp.lambdify([self.x], self.evaluate_parameters(x_symb), 'numpy'))
                         break
+    
+    def parse_expr(self, expr):
+        return sp.parse_expr(expr)
 
     def transform(self, X):
         if self.transform_functions:
@@ -558,7 +774,12 @@ class Metric():
         
         return norm'''
     
+    def g_f(self, x):
+        return lambdify([*self.x, *self.get_parameters_symb()], self.subs_functions(self.g))(*x, *self.get_parameters_val())
+
     def Christoffel(self, mu, nu, rho):
+        """The mu-nu-rho Christoffel symbol, :math:`\Gamma^{\mu}_{\nu\rho}` related to the metric tensor.
+        """
         ch = 0
         for sigma in range(4):
             ch += self.g_inv[rho,sigma]*(self.g[sigma, nu].diff(self.x[mu])+self.g[mu, sigma].diff(self.x[nu])-self.g[mu, nu].diff(self.x[sigma]))/2
